@@ -9,6 +9,7 @@
 #
 from .account import Account as Account
 from .account import AccountSlot as AccountSlot
+from .account import Valuation as Valuation
 from .accountflags import AccountFlags as AccountFlags
 from .accountinfo import AccountInfo as AccountInfo
 from .accountinfoconverter import (
@@ -37,7 +38,9 @@ from .client import FailedToFetchBlockhashException as FailedToFetchBlockhashExc
 from .client import NodeIsBehindException as NodeIsBehindException
 from .client import RateLimitException as RateLimitException
 from .client import RPCCaller as RPCCaller
-from .client import SlotHolder as SlotHolder
+from .client import AbstractSlotHolder as AbstractSlotHolder
+from .client import CheckingSlotHolder as CheckingSlotHolder
+from .client import NullSlotHolder as NullSlotHolder
 from .client import StaleSlotException as StaleSlotException
 from .client import (
     TooManyRequestsRateLimitException as TooManyRequestsRateLimitException,
@@ -105,6 +108,12 @@ from .instructions import (
 from .instructions import (
     build_mango_set_referrer_memory_instructions as build_mango_set_referrer_memory_instructions,
 )
+from .instructions import (
+    build_mango_settle_fees_instructions as build_mango_settle_fees_instructions,
+)
+from .instructions import (
+    build_mango_settle_pnl_instructions as build_mango_settle_pnl_instructions,
+)
 from .instructions import (
     build_mango_update_funding_instructions as build_mango_update_funding_instructions,
 )
@@ -249,8 +258,6 @@ from .oracle import OracleProvider as OracleProvider
 from .oracle import OracleSource as OracleSource
 from .oracle import Price as Price
 from .oracle import SupportedOracleFeature as SupportedOracleFeature
-from .orderbookside import OrderBookSideType as OrderBookSideType
-from .orderbookside import PerpOrderBookSide as PerpOrderBookSide
 from .orders import Order as Order
 from .orders import OrderType as OrderType
 from .orders import OrderBook as OrderBook
@@ -278,6 +285,7 @@ from .perpmarket import PerpMarket as PerpMarket
 from .perpmarket import PerpMarketInstructionBuilder as PerpMarketInstructionBuilder
 from .perpmarket import PerpMarketOperations as PerpMarketOperations
 from .perpmarket import PerpMarketStub as PerpMarketStub
+from .perpmarket import PerpOrderBookSide as PerpOrderBookSide
 from .perpmarketdetails import PerpMarketDetails as PerpMarketDetails
 from .perpopenorders import PerpOpenOrders as PerpOpenOrders
 from .placedorder import PlacedOrder as PlacedOrder
