@@ -16,6 +16,7 @@
 import pandas
 import typing
 
+from dataclasses import dataclass
 from decimal import Decimal
 from solana.publickey import PublicKey
 from solana.rpc.types import MemcmpOpts
@@ -30,6 +31,8 @@ from .encoding import encode_key
 from .group import Group, GroupSlot, GroupSlotPerpMarket
 from .instructions import (
     build_mango_deposit_instructions,
+    build_mango_settle_fees_instructions,
+    build_mango_settle_pnl_instructions,
     build_mango_withdraw_instructions,
 )
 from .instrumentvalue import InstrumentValue
@@ -39,6 +42,7 @@ from .observables import Disposable
 from .openorders import OpenOrders
 from .orders import Side
 from .perpaccount import PerpAccount
+from .perpmarketdetails import PerpMarketDetails
 from .perpopenorders import PerpOpenOrders
 from .placedorder import PlacedOrder
 from .tokens import Instrument, Token
@@ -167,6 +171,15 @@ class AccountSlot:
     def raw_net_value(self) -> Decimal:
         return self.raw_deposit - self.raw_borrow
 
+    def pnl(
+        self,
+        perp_market_cache: PerpMarketCache,
+        price: InstrumentValue,
+    ) -> Decimal:
+        if self.perp_account is None:
+            return Decimal(0)
+        return self.perp_account.pnl(perp_market_cache, price)
+
     def __str__(self) -> str:
         perp_account: str = "None"
         if self.perp_account is not None:
@@ -697,6 +710,201 @@ class Account(AddressableAccount):
         all_instructions = signers + create_ata + withdraw
         return all_instructions.execute(context)
 
+    def redeem_all_perp_pnl(
+        self,
+        context: Context,
+        wallet: Wallet,
+        group: Group,
+        cache: Cache,
+    ) -> typing.Sequence[str]:
+        all_accounts = Account.load_all(context, group)
+        self._logger.debug(f"Fetched {len(all_accounts)} Mango Accounts.")
+
+        instructions = CombinableInstructions.from_wallet(wallet)
+        for slot in self.slots:
+            if (slot.perp_account is not None) and (not slot.perp_account.empty):
+                self._logger.debug(
+                    f"Redeeming perp account for {slot.base_instrument.symbol}"
+                )
+                price = group.token_price_from_cache(cache, slot.base_instrument)
+                perp_market_cache = group.perp_market_cache_from_cache(
+                    cache, slot.base_instrument
+                )
+                if perp_market_cache is None:
+                    raise Exception(
+                        f"Could not load perp market cache for {slot.base_instrument.symbol} despite having a perp account for it."
+                    )
+
+                instructions += self.build_redeem_pnl_instructions_for_market(
+                    context, group, all_accounts, slot, perp_market_cache, price
+                )
+            else:
+                self._logger.debug(f"No perp account for {slot.base_instrument.symbol}")
+
+        self._logger.debug(
+            f"Have {len(instructions.instructions)} instructions to send for all perp markets."
+        )
+
+        return instructions.execute(context)
+
+    def redeem_pnl_for_perp_market(
+        self,
+        context: Context,
+        wallet: Wallet,
+        group: Group,
+        slot: AccountSlot,
+        perp_market_cache: PerpMarketCache,
+        price: InstrumentValue,
+    ) -> typing.Sequence[str]:
+        group_slot = group.slots[slot.index]
+        if group_slot.perp_market is None:
+            return []
+
+        if slot.perp_account is None:
+            return []
+
+        pnl = slot.perp_account.pnl(perp_market_cache, price)
+        if pnl == 0:
+            # No PnL to redeem
+            return []
+
+        self._logger.debug(
+            f"Trying to settle {pnl:,.8f} {group_slot.quote_token_bank.token.symbol}."
+        )
+
+        instructions = CombinableInstructions.from_wallet(wallet)
+
+        all_accounts = Account.load_all(context, group)
+        self._logger.debug(f"Fetched {len(all_accounts)} Mango Accounts.")
+
+        instructions += self.build_redeem_pnl_instructions_for_market(
+            context, group, all_accounts, slot, perp_market_cache, price
+        )
+
+        return instructions.execute(context)
+
+    def build_redeem_pnl_instructions_for_market(
+        self,
+        context: Context,
+        group: Group,
+        all_accounts: typing.Sequence["Account"],
+        slot: AccountSlot,
+        perp_market_cache: PerpMarketCache,
+        price: InstrumentValue,
+    ) -> CombinableInstructions:
+        group_slot = group.slots[slot.index]
+        if group_slot.perp_market is None:
+            self._logger.debug(f"No perp market for {slot.base_instrument.symbol}")
+            return CombinableInstructions.empty()
+
+        if slot.perp_account is None:
+            self._logger.debug(f"No perp account for {slot.base_instrument.symbol}")
+            return CombinableInstructions.empty()
+
+        pnl = slot.perp_account.pnl(perp_market_cache, price)
+        if pnl == 0:
+            # No PnL to redeem
+            return CombinableInstructions.empty()
+
+        self._logger.debug(
+            f"Trying to settle {pnl:,.8f} {group_slot.quote_token_bank.token.symbol} on {group_slot.perp_market_symbol}."
+        )
+
+        pnl_is_negative: bool = False
+        if pnl < 0:
+            pnl_is_negative = True
+
+        quote_root_bank = group.shared_quote.ensure_root_bank(context)
+
+        instructions = CombinableInstructions.empty()
+
+        # If negative, build settle fees instruction
+        if pnl_is_negative:
+            perp_market = PerpMarketDetails.load(
+                context, group_slot.perp_market.address, group
+            )
+            quote_node_bank = quote_root_bank.pick_node_bank(context)
+            self._logger.debug("Adding SettleFees instruction.")
+            instructions += build_mango_settle_fees_instructions(
+                context,
+                group,
+                perp_market,
+                self,
+                quote_root_bank,
+                quote_node_bank,
+            )
+
+        all_accounts_except_self = filter(
+            lambda acc: acc.address != self.address,
+            all_accounts,
+        )
+
+        # Filter out all accounts with zero or same-signed PnL
+        if pnl_is_negative:
+            filtered = list(
+                filter(
+                    lambda acc: acc.slots[slot.index].pnl(perp_market_cache, price) > 0,
+                    all_accounts_except_self,
+                )
+            )
+        else:
+            filtered = list(
+                filter(
+                    lambda acc: acc.slots[slot.index].pnl(perp_market_cache, price) < 0,
+                    all_accounts_except_self,
+                )
+            )
+        self._logger.debug(
+            f"Removed irrelevent accounts - now have {len(filtered)} Mango Accounts."
+        )
+
+        # Now we have a filtered list, sort it
+        sorted_accounts = sorted(
+            filtered,
+            key=lambda acc: acc.slots[slot.index].pnl(perp_market_cache, price),
+            reverse=pnl_is_negative,
+        )
+
+        # Add 5% to our collection so we're sure we're sending enough accounts even if
+        # the price has shifted. (This doesn't affect the redemption value.)
+        target_pnl = (pnl * Decimal("1.01")).copy_abs()
+
+        # Loop over top N accounts until we have redeemed enough or we run out of accounts.
+        collected = Decimal(0)
+        for other in sorted_accounts:
+            other_perp_account = other.slots[slot.index].perp_account
+            if other_perp_account is not None:
+                other_pnl = other_perp_account.pnl(perp_market_cache, price)
+
+                # Build instruction
+                instructions += build_mango_settle_pnl_instructions(
+                    context,
+                    group,
+                    group_slot,
+                    self,
+                    other,
+                    quote_root_bank,
+                )
+
+                collected += other_pnl
+                self._logger.debug(
+                    f"Collected {collected:,.8f} out of {target_pnl:,.8f} - added {other_pnl:,.8f} from {other.address}"
+                )
+
+                # Add 1% to our collection so we're sure we're sending enough accounts even if
+                # the price has shifted. (This doesn't affect the redemption value.)
+                if collected.copy_abs() > target_pnl:
+                    self._logger.debug(
+                        f"Done. Have collected {collected:,.8f} to redeem against."
+                    )
+                    break
+
+        self._logger.debug(
+            f"Have {len(instructions.instructions)} instructions to send for {group_slot.perp_market_symbol}."
+        )
+
+        return instructions
+
     def slot_by_instrument_or_none(
         self, instrument: Instrument
     ) -> typing.Optional[AccountSlot]:
@@ -884,13 +1092,15 @@ class Account(AddressableAccount):
             # --
             perp_position: Decimal = Decimal(0)
             perp_notional_position: Decimal = Decimal(0)
-            perp_value: Decimal = Decimal(0)
             perp_health_base: Decimal = Decimal(0)
             perp_health_quote: Decimal = Decimal(0)
             unsettled_funding: Decimal = Decimal(0)
             perp_health_base_value: Decimal = Decimal(0)
             perp_asset: Decimal = Decimal(0)
             perp_liability: Decimal = Decimal(0)
+            redeemable_pnl: Decimal = Decimal(0)
+            perp_base_lot_size: Decimal = Decimal(0)
+            perp_quote_lot_size: Decimal = Decimal(0)
             if (
                 slot.perp_account is not None
                 and not slot.perp_account.empty
@@ -904,13 +1114,15 @@ class Account(AddressableAccount):
                         f"Could not find perp market in Group at index {slot.index}."
                     )
 
+                perp_base_lot_size = perp_market.base_lot_size
+                perp_quote_lot_size = perp_market.quote_lot_size
+
                 perp_position = (
                     slot.perp_account.lot_size_converter.base_size_lots_to_number(
                         slot.perp_account.base_position
                     )
                 )
                 perp_notional_position = perp_position * price.value
-                perp_value = slot.perp_account.quote_position_raw
                 cached_perp_market: typing.Optional[
                     PerpMarketCache
                 ] = market_cache.perp_market
@@ -941,13 +1153,6 @@ class Account(AddressableAccount):
                 perp_bids_base_net: Decimal = perp_position + bids_quantity
                 perp_asks_base_net: Decimal = perp_position - asks_quantity
 
-                perp_asset = slot.perp_account.asset_value(
-                    cached_perp_market, price.value
-                )
-                perp_liability = slot.perp_account.liability_value(
-                    cached_perp_market, price.value
-                )
-
                 quote_pos = slot.perp_account.quote_position / (
                     10**self.shared_quote_token.decimals
                 )
@@ -967,6 +1172,15 @@ class Account(AddressableAccount):
                     )
                 perp_health_base_value = perp_health_base * price.value
 
+                perp_asset = slot.perp_account.asset_value(
+                    cached_perp_market, price.value
+                )
+                perp_liability = slot.perp_account.liability_value(
+                    cached_perp_market, price.value
+                )
+
+                redeemable_pnl = slot.perp_account.pnl(cached_perp_market, price)
+
             group_slot: typing.Optional[GroupSlot] = None
             if market_cache is not None:
                 group_slot = group.slot_by_instrument(slot.base_instrument)
@@ -1058,7 +1272,7 @@ class Account(AddressableAccount):
                 "PerpHealthQuote": perp_health_quote,
                 "PerpAsset": perp_asset,
                 "PerpLiability": perp_liability,
-                "PerpValue": perp_value,
+                "RedeemablePnL": redeemable_pnl,
                 "UnsettledFunding": unsettled_funding,
                 "SpotInitAssetWeight": spot_init_asset_weight,
                 "SpotMaintAssetWeight": spot_maint_asset_weight,
@@ -1068,6 +1282,10 @@ class Account(AddressableAccount):
                 "PerpMaintAssetWeight": perp_maint_asset_weight,
                 "PerpInitLiabilityWeight": perp_init_liab_weight,
                 "PerpMaintLiabilityWeight": perp_maint_liab_weight,
+                "BaseDecimals": slot.base_instrument.decimals,
+                "QuoteDecimals": group.shared_quote_token.decimals,
+                "PerpBaseLotSize": perp_base_lot_size,
+                "PerpQuoteLotSize": perp_quote_lot_size,
             }
             asset_data += [data]
         frame: pandas.DataFrame = pandas.DataFrame(asset_data)
@@ -1192,6 +1410,10 @@ class Account(AddressableAccount):
             return Decimal(0)
         return -liabilities / (assets + liabilities)
 
+    def redeemable_pnl(self, frame: pandas.DataFrame) -> InstrumentValue:
+        value: Decimal = frame["RedeemablePnL"].sum()
+        return InstrumentValue(self.shared_quote_token, value)
+
     def __str__(self) -> str:
         info = f"'{self.info}'" if self.info else "(un-named)"
         shared_quote: str = f"{self.shared_quote}".replace("\n", "\n        ")
@@ -1223,3 +1445,87 @@ class Account(AddressableAccount):
 
     def __repr__(self) -> str:
         return f"{self}"
+
+
+@dataclass
+class Valuation:
+    account: Account
+    open_orders: typing.Dict[str, OpenOrders]
+    frame: pandas.DataFrame
+    total_value: InstrumentValue
+
+    @property
+    def init_health(self) -> InstrumentValue:
+        return self.account.init_health(self.frame)
+
+    @property
+    def maint_health(self) -> InstrumentValue:
+        return self.account.maint_health(self.frame)
+
+    @property
+    def init_health_ratio(self) -> Decimal:
+        return self.account.init_health_ratio(self.frame)
+
+    @property
+    def maint_health_ratio(self) -> Decimal:
+        return self.account.maint_health_ratio(self.frame)
+
+    @property
+    def is_liquidatable(self) -> bool:
+        return self.account.is_liquidatable(self.frame)
+
+    @property
+    def leverage(self) -> Decimal:
+        return self.account.leverage(self.frame)
+
+    @property
+    def redeemable_pnl(self) -> InstrumentValue:
+        return self.account.redeemable_pnl(self.frame)
+
+    @staticmethod
+    def create(
+        account: Account,
+        group: Group,
+        open_orders_data: typing.Dict[str, AccountInfo],
+        cache: Cache,
+    ) -> "Valuation":
+        oos = {}
+        for slot in account.slots:
+            if slot.spot_open_orders is not None:
+                oo = OpenOrders.parse(
+                    open_orders_data[str(slot.spot_open_orders)],
+                    Token.ensure(slot.base_instrument),
+                    group.shared_quote_token,
+                )
+                oos[str(oo.address)] = oo
+        frame = account.to_dataframe(group, oos, cache)
+        value = account.total_value(frame)
+        return Valuation(account, oos, frame, value)
+
+    def __lt__(self, other: "Valuation") -> bool:
+        return self.total_value.value < other.total_value.value
+
+    def __le__(self, other: "Valuation") -> bool:
+        return self.total_value.value <= other.total_value.value
+
+    def __gt__(self, other: "Valuation") -> bool:
+        return self.total_value.value > other.total_value.value
+
+    def __ge__(self, other: "Valuation") -> bool:
+        return self.total_value.value >= other.total_value.value
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, Valuation):
+            return self.total_value.value == other.total_value.value
+        return False
+
+    def __ne__(self, other: object) -> bool:
+        if isinstance(other, Valuation):
+            return self.total_value.value != other.total_value.value
+        return True
+
+    def __str__(self) -> str:
+        return f"""« Valuation {str(self.account.address):<44} {self.total_value.value:>25,.8f} {self.total_value.token.symbol} »"""
+
+    def __repr__(self) -> str:
+        return f"{self}"
