@@ -234,6 +234,18 @@ class SerumEventQueue(AddressableAccount):
 
         return distinct
 
+    @property
+    def events(self) -> typing.Sequence[SerumEvent]:
+        return [*self.processed_events, *self.unprocessed_events]
+
+    @property
+    def fills(self) -> typing.Sequence[SerumEvent]:
+        fills: typing.List[SerumEvent] = []
+        for event in self.events:
+            if event.event_flags.fill:
+                fills += [event]
+        return fills
+
     @property
     def capacity(self) -> int:
         return len(self.unprocessed_events) + len(self.processed_events)
